<?php
/**
 * A UniversalNodeLocator discovery class
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2015 Hub Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class UniversalNodeLocatorDiscovery extends BaseNodeDiscovery implements DiscoverableUniversalNodeLocator, Registerable {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Create an instance of this class
	 *
	 * @return	$discoveryInstance	An instance of this discovery class
	 */
	public static final function createUniversalNodeLocatorDiscovery () {
		// Get an instance of this class
		$discoveryInstance = new UniversalNodeLocatorDiscovery();

		// Return the prepared instance
		return $discoveryInstance;
	}

	/**
	 * "Discovers" an instance of a LocateableNode class for given NodeHelper class
	 *
	 * @param	$nodeInstance	An instance of a NodeHelper class
	 * @return	$unlInstance	An instance of a LocateableNode class
	 */
	public function discoverUniversalNodeLocatorByNode (NodeHelper $nodeInstance) {
		/*
		 * First get an instance from the configured hub communication protocol
		 * type (which is mostly TCP, so you get a TcpProtocolResolver here).
		 */
		$resolverInstance = ProtocolResolverFactory::createResolverFromConfiguredProtocol();

		// Then resolve the node instance into an UNL instance
		$unlInstance = $resolverInstance->resolveUniversalNodeLocatorFromNodeHelper($nodeInstance);

		// ... and return it
		return $unlInstance;
	}

	/**
	 * "Discovers" the full universal node locator for given configuration entry
	 * which should represent the address part of the UNL. In case of an IP
	 * address, this *must not* include the port number as this is being found
	 * out for you.
	 *
	 * Valid values for $configKey: internal,external
	 *
	 * @param	$configKey	Key for address to get (valid: internal,external)
	 * @return	$unl		Universal node locator
	 */
	public function discoverUniversalNodeLocatorByConfiguredAddress ($configKey) {
		// Debug message
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('UNL-DISCOVERY[' . __METHOD__ . ':' . __LINE__ . ']:configKey=' . $configKey . ' - CALLED!');

		// Is there cache?
		if (!isset($GLOBALS[__METHOD__][$configKey])) {
			// Validate config key
			assert(($configKey == 'internal') || ($configKey == 'external'));

			/*
			 * First get an instance from the configured hub communication protocol
			 * type (which is mostly TCP, so you get a TcpProtocolResolver here).
			 */
			$resolverInstance = ProtocolResolverFactory::createResolverFromConfiguredProtocol();

			// Get back the full UNL
			$GLOBALS[__METHOD__][$configKey] = $resolverInstance->resolveUniversalNodeLocatorFromConfigKey($configKey);
		} // END - if

		// Return cached value
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('UNL-DISCOVERY[' . __METHOD__ . ':' . __LINE__ . ']:configKey=' . $configKey . ',unl=' . $GLOBALS[__METHOD__][$configKey] . ' - EXIT!');
		return $GLOBALS[__METHOD__][$configKey];
	}
}

// [EOF]
?>
