<?php
/**
 * A factory class for fragmenter
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2015 Hub Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class FragmenterFactory extends ObjectFactory {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Returns a singleton fragmenter instance. If no instance is found, it fill
	 * be generated and stored in registry, else the fragmenter from the
	 * registry will be returned.
	 *
	 * @param	$fragmenterType			The fragmenter's type
	 * @return	$fragmenterInstance		A fragmenter instance
	 */
	public static final function createFragmenterInstance ($fragmenterType) {
		// If there is no fragmenter?
		if (Registry::getRegistry()->instanceExists($fragmenterType . '_fragmenter')) {
			// Get fragmenter from registry
			$fragmenterInstance = Registry::getRegistry()->getInstance($fragmenterType . '_fragmenter');
		} else {
			// Get the fragmenter instance
			$fragmenterInstance = self::createObjectByConfiguredName($fragmenterType . '_fragmenter_class');

			// Add it to the registry
			Registry::getRegistry()->addInstance($fragmenterType . '_fragmenter', $fragmenterInstance);
		}

		// Return the instance
		return $fragmenterInstance;
	}
}

// [EOF]
?>
