<?php
/**
 * A DHT recipient
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2015 Hub Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
class DhtRecipient extends BaseRecipient implements Recipient {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Creates an instance of this class
	 *
	 * @return	$recipientInstance		An instance of a Recipient class
	 */
	public final static function createDhtRecipient () {
		// Get new instance
		$recipientInstance = new DhtRecipient();

		// Return the prepared instance
		return $recipientInstance;
	}

	/**
	 * Tries to resolve given recipient into session ids or Universal Node Locator
	 * depending on implementation (hint: Template Method Pattern). This
	 * implementation will add more than one recipient to the list as a DHT is
	 * distributed and the package might go to more than one recipient.
	 *
	 * @param	$recipient		Recipient to resolve (e.g. could be a virtual recipient or direct session id)
	 * @param	$listInstance	An instance of a Listable class
	 * @param	$packageData	Valid package data array
	 * @return	void
	 * @throws	FrameworkException	Could throw different exceptions depending on implementation
	 */
	public function resolveRecipient ($recipient, Listable $listInstance, array $packageData) {
		// Debug message
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('DHT-RECIPIENT[' . __METHOD__ . ':' . __LINE__ . ']: recipient=' . $recipient . ' - CALLED!');

		// Make sure the recipient is valid
		assert($recipient == NetworkPackage::NETWORK_TARGET_DHT);

		// Get recipient discovery instance
		$discoverInstance = ObjectFactory::createObjectByConfiguredName('dht_recipient_discovery_class');

		// "Discover" recipients by given package data
		$recipients = $discoverInstance->resolveRecipientsByPackageData($packageData);

		// Now "walk" through all elements and add them to the list
		foreach ($recipients as $recipient) {
			// These array elements must exist for this loop:
			// @TODO Unfinished
			die(__METHOD__ . ':recipient=' . print_r($recipient, TRUE));
			assert(!empty($recipient[NodeDistributedHashTableDatabaseWrapper::DB_COLUMN_EXTERNAL_ADDRESS]));

			// Put ip and port both together
			$unl = $recipient[NodeDistributedHashTableDatabaseWrapper::DB_COLUMN_EXTERNAL_ADDRESS];

			// Add it to the list
			//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('DHT-RECIPIENT[' . __METHOD__ . ':' . __LINE__ . ']: Calling listInstance->addEntry(' . $unl . ') ...');
			$listInstance->addEntry('unl', $unl);
		} // END - foreach

		// Debug message
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('DHT-RECIPIENT[' . __METHOD__ . ':' . __LINE__ . ']: recipient=' . $recipient . ' - EXIT!');
	}
}

// [EOF]
?>
